/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
 Header:       FGFDMExecNET.h
 Author:       Lorne McIntosh
 Date started: 2007/07/31

--------- Copyright (C) 2007  Lorne D. McIntosh (lorne@lorneswork.com) ---------

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

HISTORY
--------------------------------------------------------------------------------
2007/07/31   LDM   Created

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/

#pragma once

#include "FGFDMExec.h"
#include "FGState.h"
#include "StringConverter.h"

using namespace System;

namespace JSBSimNET
{
	/// <summary>Wrapper for FGFDMExec</summary>
	public ref class FGFDMExecNET
	{
		public:

			/// <summary>The FGFDMExecNET class constructor</summary>
			FGFDMExecNET();

			/// <summary>The FGFDMExecNET class destructor</summary>
			~FGFDMExecNET();
			
			/// <summary>This function executes each scheduled model in succession.</summary>
			/// <returns>Returns true if successful, false if sim should be ended</returns>
			bool Run(void);

			/// <summary>
			/// Initializes the sim from the initial condition object and executes
			/// each scheduled model without integrating i.e. dt=0.
			/// </summary>
			/// <returns>Returns true if successful, false otherwise</returns>
			bool RunIC(void);
			
			/// <summary>Loads an aircraft model.</summary>
			/// <param name='AircraftPath'>Path to the aircraft/ directory. For instance:
			/// "aircraft". Under aircraft, then, would be directories for various
			/// modeled aircraft such as C172/, x15/, etc.</param>
			/// <param name='EnginePath'>Path to the directory under which engine config
			/// files are kept, for instance "engine"</param>
			/// <param name='model'>The name of the aircraft model itself. This file will
			/// be looked for in the directory specified in the AircraftPath variable,
			/// and in turn under the directory with the same name as the model. For
			/// instance: "aircraft/x15/x15.xml</param>
			/// <param name='addModelToPath'>set to true to add the model name to the AircraftPath</param>
			/// <returns>Returns true if successful, false otherwise</returns>
			bool LoadModel(String^ AircraftPath, String^ EnginePath, String^ model, bool addModelToPath);
			
			/// <summary>Loads an aircraft model.</summary>
			/// <param name='AircraftPath'>Path to the aircraft/ directory. For instance:
			/// "aircraft". Under aircraft, then, would be directories for various
			/// modeled aircraft such as C172/, x15/, etc.</param>
			/// <param name='EnginePath'>Path to the directory under which engine config
			/// files are kept, for instance "engine"</param>
			/// <param name='model'>The name of the aircraft model itself. This file will
			/// be looked for in the directory specified in the AircraftPath variable,
			/// and in turn under the directory with the same name as the model. For
			/// instance: "aircraft/x15/x15.xml</param>
			/// <returns>Returns true if successful, false otherwise</returns>
			bool LoadModel(String^ AircraftPath, String^ EnginePath, String^ model);
			
			/// <summary>
			/// Loads an aircraft model. The paths to the aircraft and engine
			/// config file directories must be set prior to calling this.
			/// </summary>
			/// <param name='model'>The name of the aircraft model itself. This file will
			/// be looked for in the directory specified in the AircraftPath variable,
			/// and in turn under the directory with the same name as the model. For
			/// instance: "aircraft/x15/x15.xml</param>
			/// <param name='addModelToPath'>set to true to add the model name to the AircraftPath</param>
			/// <returns>Returns true if successful, false otherwise</returns>
			bool LoadModel(String^ model, bool addModelToPath);
			
			/// <summary>
			/// Loads an aircraft model. The paths to the aircraft and engine
			/// config file directories must be set prior to calling this.
			/// </summary>
			/// <param name='model'>The name of the aircraft model itself. This file will
			/// be looked for in the directory specified in the AircraftPath variable,
			/// and in turn under the directory with the same name as the model. For
			/// instance: "aircraft/x15/x15.xml</param>
			/// <returns>Returns true if successful, false otherwise</returns>
			bool LoadModel(String^ model);
			
			/// <summary>Loads a script</summary>
			/// <param name='Script'>The full path name and file name for the script to be loaded</param>
			/// <returns>Returns true if successful, false otherwise</returns>
			bool LoadScript(String^ Script);

			/// <summary>
			/// Loads the initial conditions.
			/// This isn't in the original FGFDMExec - it's been added to simplify the wrapping.
			/// </summary>
			/// <param name='rstname'>The name of an initial conditions file</param>
			/// <param name='useStoredPath'>True if the stored path to the IC file should be used</param>
			/// <returns>Returns true if successful, false otherwise</returns>
			bool LoadIC(String^ rstname, bool useStoredPath);
			
			/// <summary>
			/// Loads the initial conditions.
			/// This isn't in the original FGFDMExec - it's been added to simplify the wrapping.
			/// </summary>
			/// <param name='rstname'>The name of an initial conditions file</param>
			/// <returns>Returns true if successful, false otherwise</returns>
			bool LoadIC(String^ rstname);

			/// <summary>Executes trimming in the selected mode.</summary>
			/// <param name='mode'>Specifies how to trim:
			///		<list type="bullet">
			///		<item><description>0 = Longitudinal</description></item>
			///		<item><description>1 = Full</description></item>
			///		<item><description>2 = Ground</description></item>
			///		<item><description>3 = Pullup</description></item>
			///		<item><description>4 = Custom</description></item>
			///		<item><description>5 = Turn</description></item>
			///		<item><description>6 = None</description></item>
			///		</list>
			/// </param>
			void DoTrim(int mode);

			/// <summary>Disables data logging to all outputs.</summary>
			void DisableOutput(void);

			/// <summary>Enables data logging to all outputs.</summary>
			void EnableOutput(void);

			/// <summary>Pauses execution by preventing time from incrementing.</summary>
			void Hold(void);

			/// <summary>Resumes execution from a "Hold".</summary>
			void Resume(void);

			/// <summary>Returns true if the simulation is Holding (i.e. simulation time is not moving).</summary>
			bool Holding(void);

			/// <summary>
			/// Suspends the simulation and sets the delta T to zero.
			/// This isn't in the original FGFDMExec - it's been added to simplify the wrapping.
			/// </summary>
			void SuspendIntegration(void);

			/// <summary>
			/// Resumes the simulation by resetting delta T to the correct value.
			/// This isn't in the original FGFDMExec - it's been added to simplify the wrapping.
			/// </summary>
			void ResumeIntegration(void);

			/// <summary>
			/// Returns the simulation suspension state.
			/// This isn't in the original FGFDMExec - it's been added to simplify the wrapping.
			/// </summary>
			/// <returns>true if suspended, false if executing</returns>
			bool IntegrationSuspended(void);

			/// <summary>Sets the debug level.</summary>
			void SetDebugLevel(int level);

			/// <summary>Retrieves the value of a property.</summary>
			/// <param name='propertyName'>Property name of the property (ie. "position/h-sl-ft")</param>
			/// <returns>The value of the specified property</returns>
			double GetPropertyValue(String^ propertyName);
				
			/// <summary>Sets a property value.</summary>
			/// <param name='propertyName'>Property name of the property to be set (ie. "position/h-sl-ft")</param>
			/// <param name='value'>The value to set the property to</param>
			/// <returns>Returns true if the assignment succeeded, false otherwise.</returns>
			bool SetPropertyValue(String^ propertyName, double value);

			/// <summary>Represents the path to the engine config file directory</summary>
			/// <value>gets/sets the EnginePath</value>
			property String^ EnginePath
			{
				String^ get();
				void set(String^ EnginePath);
			}

			/// <summary>Represents the path to the aircraft config file directory</summary>
			/// <value>gets/sets the AircraftPath</value>
			property String^ AircraftPath
			{
				String^ get();
				void set(String^ AircraftPath);
			}

			/// <summary>Represents the full aircraft path</summary>
			/// <value>gets the AircraftPath</value>
			property String^ FullAircraftPath
			{
				String^ get();
			}
			
			/// <summary>Represents the name of the loaded model</summary>
			/// <value>gets the ModelName</value>
			property String^ ModelName
			{
				String^ get();
			}

			/// <summary>Represents the elapsed simulation time (in seconds)</summary>
			/// <value>gets the SimTime</value>
			property double SimTime
			{
				double get();
			}

			/// <summary>Represents the current frame time (delta T)</summary>
			/// <value>gets the DeltaT</value>
			property double DeltaT
			{
				double get();
			}

		private:
			/// <summary>Points to an instance of the unmanaged FGFDMExec class</summary>
			JSBSim::FGFDMExec* jsbsim;
	};
}
